		list	p=16F630           ; list directive to define processor
        #include	<p16F630.inc>
        errorlevel -302			; suppress "register not in bank0, check page bits" message.	
;
		RADIX	dec

; Timing is OK for 4MHz clocking for other speeds check TRW-2.4G datasheet

; DAT must be LSB, otherwise, code must be modified
#define DAT	PORTA,0
#define DATPORT PORTA
#define DATT	TRISA,0
#define SCK		PORTA,1
#define CE		PORTA,2
#define	DR		PORTA,3
#define CS		PORTA,4

; Packet lenght in bytes
#define TRW_DATA_LEN	24
; Address lenght in bytes
#define TRW_ADDRESS_LEN 5



trwvars	UDATA_SHR
dat		RES 1
bitcntr	RES 1
bytcntr	RES 1
addrptr	RES 1
; MUST BE CONTIGUOUS
txbuffer
address	RES TRW_ADDRESS_LEN
message	RES TRW_DATA_LEN



trwcode	CODE

; Send  a byte
; Call with data in W
trw_write_byte:
		movwf dat				; Store data
		movlw 8					; 8 bits
		movwf bitcntr
		bsf STATUS,RP0 			; Bank 1
        bcf DATT				; DATA = output
		bcf STATUS,RP0 			; Bank 0
twbl1	rlf dat,f				; C = MSB (shift bit for output)
		bcf SCK					; CLK = 0
		bsf DAT					; DATA = 1 (guess)
		btfss STATUS,C			; if C, we've guessed right
		bcf DAT					; otherwise DATA = 0
		nop
		bsf SCK					; CLK = 1
		decfsz bitcntr,f
		goto twbl1
		bcf SCK					; CLK = 0
		bsf STATUS,RP0 			; Bank 1
        bsf DATT				; DATA = input
		bcf STATUS,RP0 			; Bank 0
		return

; Receive a byte
; Returns data in W
trw_read_byte:
		clrf dat				; DATA = 0
		movlw 8					; 8 bits
		movwf bitcntr
trbl1	bsf SCK					; CLK = 1
		nop
		rrf DATPORT,W			; read data in C
		rlf dat,f				; and into register	(MSB first)	
		bcf SCK					; CLK = 0
		decfsz bitcntr,f
		goto trbl1
		movf dat,W
		return

; Initialize TRW-2.4G module
trw_init:
		bsf CS					; CS = 1
		clrf bytcntr
til1	movlw HIGH getconfig	; @#$%~ !!!
		movwf PCLATH
		movf bytcntr,W
		call getconfig			; get byte from table in W
		call trw_write_byte		; send byte in W
		incf bytcntr,f
		movlw 18				; 18 bytes
		subwf bytcntr,W
		btfss STATUS,Z
		goto til1 				; repeat
		bcf CS					; CS = 0
		return

; Set comms channel (Rx=off)
; Call with channel in W
trw_setch:
		movf dat,W				; channel *= 2
		addwf dat,W
		bsf CS					; CS = 1
		call trw_write_byte		; send byte in W
		bcf CS					; CS = 0
		return

; Enable Tx
trw_settx:
		bcf CE					; CE = 0
		nop
		bsf CS					; CS = 1
		bsf STATUS,RP0 			; Bank 1
        bcf DATT				; DATA = output
		bcf STATUS,RP0 			; Bank 0
		bcf DAT					; DATA = 0
		nop
		nop
sndbit	bsf SCK					; CLK = 1
		nop
		bcf SCK					; CLK = 0
		bsf STATUS,RP0 			; Bank 1
        bsf DATT				; DATA = input
		bcf STATUS,RP0 			; Bank 0
		bcf CS					; CS = 0
		nop
		bsf CE					; CE = 1
		return

; Enable Rx
trw_setrx:
		bcf CE					; CE = 0
		nop
		bsf CS					; CS = 1
		bsf STATUS,RP0 			; Bank 1
        bcf DATT				; DATA = output
		bcf STATUS,RP0 			; Bank 0
		bsf DAT					; DATA = 1
		goto sndbit

; Send a message to the remote end
; Call with address in address buffer and packet in packet buffer
trw_sendpacket:
		call trw_settx
		movlw (TRW_ADDRESS_LEN+TRW_DATA_LEN)	; # bytes
		movwf bytcntr			; Init # bytes to put
		movlw txbuffer			; Initialize pointer
		movwf FSR
tsl1	movf INDF,W				; get byte in W
		call trw_write_byte		; send byte in W
		incf FSR,f				; next byte
		decfsz bytcntr,f		; loop
		goto tsl1
		bcf CE					; CE = 0
		return


; Get a message from a remote end
; Returns:
;	W=0 => no msg waiting
;	W=0xFF => msg in packet buffer
trw_getpacket:
		btfss DR				; packet ?
		retlw 0					; NO
		bcf CE					; CE = 0
		movlw TRW_DATA_LEN		; # bytes
		movwf bytcntr			; Init # bytes to put
		movlw message			; Initialize pointer
		movwf FSR
tgl1	call trw_read_byte		; get byte in W
		movwf INDF				; store byte in W in buffer
		incf FSR,f				; next byte
		decfsz bytcntr,f		; loop
		goto tgl1
		retlw 0xFF

; Set remote address
; Call with remote address' index in W
trw_address:
		movwf bytcntr			; save W
		movlw HIGH addresses	; @#$%~ !!!
		movwf PCLATH
		movf bytcntr,W
		call addresses			; Get memory address for requested module's address
		movwf addrptr			; save it
		clrf bytcntr
		movlw address			; Initialize pointer
		movwf FSR
tal1	movlw HIGH addresses	; @#$%~ !!!
		movwf PCLATH
		call getaddr			; Get address byte in W
		movwf INDF				; store byte in W in buffer
		incf FSR,f				; next byte
		incf addrptr,f
		incf bytcntr,f			; keep byte count
		movlw TRW_ADDRESS_LEN	; # bytes
		subwf bytcntr,W
		btfss STATUS,Z
		goto tal1				; loop
		return

		

trwdata	CODE 0x200
; This works because AT THIS ADDRESS there is no possible overflow
; MUST preload PCLATH when address is higher than 0xFF
; All addresses + config data must fit within a 0x100 space.
; Base address must be 00, otherwise code must be modified
getaddr movf addrptr,W
addresses:
		addwf PCL,f
		retlw LOW (addrs1-2)	; 2 bytes offset when adding to PCL 
;		retlw LOW (addrs2-2)
; ...		
addrs1	retlw 0x21
		retlw 0x43
		retlw 0x65
		retlw 0x87
		retlw 0xA9
; addrs2
; ...
getconfig:
		addwf PCL,f
		retlw 0x8E
		retlw 0x08
		retlw 0x1C
		retlw 0xC0
		retlw 0xC0
		retlw 0xBB
		retlw 0xBB
		retlw 0xBB
		retlw 0xBB
		retlw 0xBB
		retlw 0x13			; LOCAL ADDRESS
		retlw 0x25			;
		retlw 0x46			;
		retlw 0x79			;
		retlw 0x8A			;
		retlw 0xA3
		retlw 0x4F
		retlw 0x14

		GLOBAL trw_sendpacket,trw_getpacket,trw_setrx,trw_setch,trw_init,trw_address
		GLOBAL message

		END
